/*
 * ATtiny OLED real time clock
 * Burn bootloader set to 4MHz(internal)
 * Created: 24/10/2024
 *  Author: moty22.co.uk
 */ 

#include <TinyI2CMaster.h>

#define modePB 4 //pin3 
#define incPB 3//pin2
#define sqw 1   //pin6
#define yc 0   //clock Y
#define yd 6   //date Y

const unsigned char  font[] = {
  0x3E, 0x41, 0x41, 0x41, 0x3E,   //new 0    
  0x00, 0x42, 0x7F, 0x40, 0x00,// 1
  0x42, 0x61, 0x51, 0x49, 0x46,// 2
  0x21, 0x41, 0x45, 0x4B, 0x31,// 3
  0x18, 0x14, 0x12, 0x7F, 0x10,// 4
  0x27, 0x45, 0x45, 0x45, 0x39,// 5
  0x3C, 0x4A, 0x49, 0x49, 0x30,// 6
  0x01, 0x71, 0x09, 0x05, 0x03,// 7
  0x36, 0x49, 0x49, 0x49, 0x36,// 8
  0x06, 0x49, 0x49, 0x29, 0x1E,// 9
  0x00, 0x00, 0x24, 0x00, 0x00,// : small
  0x20, 0x10, 0x08, 0x04, 0x02,// /
  0x40, 0x40, 0x40, 0x40, 0x40,// _
  0x01, 0x01, 0x01, 0x01, 0x01,    // line top      
  0x00, 0x00, 0x00, 0x00, 0x00// (space) 
};

unsigned char addr=0x3C;  //oled
unsigned int addr2=0x68;  //clock D0
unsigned char dly=2, il, ir, page=0;
unsigned char sec, minute, hour, minuteS, hourS;  //
unsigned char week, day, month, year, dayS, monthS, yearS;
unsigned char minuteL, minuteH, hourL, hourH, modeC=0;  //
unsigned char dayL, dayH, monthL, monthH, yearL, yearH;

void setup() {
pinMode(modePB, INPUT_PULLUP);
pinMode(incPB, INPUT_PULLUP);
pinMode(sqw, INPUT);

    TinyI2C.init();
    oled_init();   // oled init
    clrScreen();       // clear screen
    drawChar4(10, yc, 32);   //:
    drawChar4(10, yc, 79);
    drawChar2(11, yd, 3);   //"/"
    drawChar2(11, yd, 6);   //"/"

    setTime(0x0e,0);    //set SQW to 1Hz
    delay(1000);
}

void loop()
{
    if(!digitalRead(modePB)){
        delay(1000);
        ++modeC;
        if(modeC>5){modeC=0;}
        if(modeC==0){drawChar2(14, 4, 7);}//
        else if(modeC==1){drawChar2(13, 4, 4);}
        else if(modeC==2){drawChar2(14, 4, 4); drawChar2(13, 4, 1);}
        else if(modeC==3){drawChar2(12, 4, 1);}
        else if(modeC==4){drawChar2(14, 4, 1); drawChar2(12, 4, 4);}
        else if(modeC==5){drawChar2(14, 4, 4); drawChar2(12, 4, 7);}
    }
    
    if(modeC==0){
        while(digitalRead(sqw)){}
        while(!digitalRead(sqw)){}   //read time when sqw is low.

        readTime(0);

        drawChar4((sec>>4 & 7), yc, 94);
        drawChar4((sec & 15), yc, 111);
        minuteH=minute>>4 & 7;
        minuteL=minute & 15;
        drawChar4(minuteH, yc, 47);
        drawChar4(minuteL, yc, 64);
        hourL=hour & 15;
        hourH=hour>>4 & 3;
        drawChar4(hourH, yc, 0);
        drawChar4(hourL, yc, 17);
        dayL=day & 15;
        dayH=day>>4 & 3;
        drawChar2(dayH, yd, 1);
        drawChar2(dayL, yd, 2);
        monthL=month & 15;
        monthH=month>>4 & 1;
        drawChar2(monthH, yd, 4);
        drawChar2(monthL, yd, 5);
        yearL=year & 15;
        yearH=year>>4 & 15;
        drawChar2(yearH, yd, 7);
        drawChar2(yearL, yd, 8);
    }
//minuteute setting  
    if(modeC==1 && !digitalRead(incPB)){
        delay(500);
        ++minuteL;
        if(minuteL>9){minuteL=0; ++minuteH;} 
        if(minuteH>5){minuteH=0;}    
        drawChar4(minuteH, yc, 47);
        drawChar4(minuteL, yc, 64);
        minuteS=0;
        minuteS=(minuteH<<4) + minuteL;
        setTime(1,minuteS);
    }
//hour setting
    if(modeC==2 && !digitalRead(incPB)){
        delay(500);
        ++hourL;
        if(hourL>9){hourL=0; ++hourH;}
        if((hourH==2) && (hourL>3)){hourH=0; hourL=0;}
        drawChar4(hourH, yc, 0);
        drawChar4(hourL, yc, 17);
        hourS=0;
        hourS=(hourH<<4) + hourL;
        setTime(2,hourS);
    }            
//day setting        
    if(modeC==3 && !digitalRead(incPB)){
        delay(500);
        ++dayL;
        if(dayL>9){dayL=0; ++dayH;}
        if(dayH==3 && dayL>1){dayH=0; dayL=0;}    
        drawChar2(dayH, yd, 1);
        drawChar2(dayL, yd, 2);
        dayS=0;
        dayS=(dayH<<4) + dayL;
        setTime(4,dayS);
    }
//month setting        
    if(modeC==4 && !digitalRead(incPB)){
        delay(500);
        ++monthL;
        if(monthL>9){monthL=0; ++monthH;}
        if(monthH==1 && monthL>2){monthH=0; monthL=0;}    
        drawChar2(monthH, yd, 4);
        drawChar2(monthL, yd, 5);
        monthS=0;
        monthS=(monthH<<4) + monthL;
        setTime(5,monthS);
    }
//year setting        
    if(modeC==5 && !digitalRead(incPB)){
        delay(500);
        ++yearL;
        if(yearL>9){yearL=0; ++yearH;}                
        if(yearH>9){yearH=0;}    
        drawChar2(yearH, yd, 7);
        drawChar2(yearL, yd, 8);
        yearS=0;
        yearS=(yearH<<4) + yearL;
        setTime(6,yearS);
    }                    

}

void readTime (unsigned char reg){
  TinyI2C.start(addr2, 0);
  TinyI2C.write(reg);
  TinyI2C.start(addr2, -1);
  sec = TinyI2C.read();
  minute = TinyI2C.read();
  hour = TinyI2C.read();
  week = TinyI2C.read();
  day = TinyI2C.read();
  month = TinyI2C.read();
  year = TinyI2C.readLast();
  TinyI2C.stop();

}

void setTime (unsigned char reg, unsigned char setting){
  TinyI2C.start(addr2, 0);
  TinyI2C.write(reg);
  TinyI2C.write(setting);
  TinyI2C.stop();

} 

void drawChar4(char fig, unsigned char y, unsigned char x)  //size Hx4 Wx3
{
    unsigned char i, j, line, b1, b2, b3, b4;    //
    
    command(0x20);    // vert mode
    command(0x01);

    command(0x21);     //col addr
    command(x); //col start
    command(x + 14);  //col end
    command(0x22);    //0x22
    command(y); // Page start
    command(y+3); // Page end
    
    TinyI2C.start(addr, 0);            // address
    TinyI2C.write(0x40);

    for (i = 0; i < 5; i++){
        line=font[5*(fig)+i];
        b1=0; b2=0; b3=0; b4=0;
            // expend char    
        if(line & 64) {b4 +=240;}
        if(line & 32) {b4 +=15;}
        if(line & 16) {b3 +=240;}           
        if(line & 8) {b3 +=15;}
        
        if(line & 4) {b2 +=240;}
        if(line & 2) {b2 +=15;}
        if(line & 1) {b1 +=240;}
        for(j=0;j<3;j++){
         TinyI2C.write(b1); //top page
         TinyI2C.write(b2);  //second page
         TinyI2C.write(b3);
         TinyI2C.write(b4);
        } 
    }
    TinyI2C.stop();
        
    command(0x20);      // horizontal mode
    command(0x00);    
        
}

void command( unsigned char comm){
    
    TinyI2C.start(addr, 0);
    TinyI2C.write(0);
    TinyI2C.write(comm);             // command code
    TinyI2C.stop();
}

void oled_init() {
    
    command(0xAE);   // DISPLAYOFF
    command(0x8D);         // CHARGEPUMP *
    command(0x14);     //0x14-pump on
    command(0x20);         // MEMORYMODE
    command(0x0);      //0x0=horizontal, 0x01=vertical, 0x02=page
    command(0xA1);        //SEGREMAP * A0/A1=top/bottom 
    command(0xC8);     //COMSCANDEC * C0/C8=left/right
    command(0xDA);         // SETCOMPINS *
    command(0x12);   //0x22=4rows, 0x12=8rows
    command(0x81);        // SETCONTRAST
    command(0x9F);     //0x8F
//    command(0xD5);  //SETDISPLAYCLOCKDIV 
//    command(0x80);  
//    command(0xA8);       // SETMULTIPLEX
//    command(0x3F);     //0x1F
//    command(0xD3);   // SETDISPLAYOFFSET
//    command(0x0);  
//    command(0x40); // SETSTARTLINE  
//    command(0xD9);       // SETPRECHARGE
//    command(0xF1);
//    command(0xDB);      // SETVCOMDETECT
//    command(0x40);
//    command(0xA4);     // DISPLAYALLON_RESUME
 //   command(0xA6);      // NORMALDISPLAY
    command(0xAF);          //DISPLAYON

}

void clrScreen()    //fill screen with 0
{
    unsigned char y, i;
    
    for ( y = 0; y < 8; y++ ) {
    command(0xB0 + y);     //0 to 7 pages
    command(0x00); //low nibble
    command(0x10);  //high nibble
    
        TinyI2C.start(addr, 0);            // address
        TinyI2C.write(0x40);
        for (i = 0; i < 128; i++){
             TinyI2C.write(0x0);
        }
        TinyI2C.stop();
    }    
}
    //size 2 chars
void drawChar2(char fig, unsigned char y, unsigned char x)
{
    unsigned char i, line, btm, top;    //
    
    command(0x20);    // vert mode
    command(0x01);

    command(0x21);     //col addr
    command(13 * x); //col start
    command(13 * x + 9);  //col end
    command(0x22);    //0x22
    command(y); // Page start
    command(y+1); // Page end
    
    TinyI2C.start(addr, 0);            // address
    TinyI2C.write(0x40);

    for (i = 0; i < 5; i++){
      line=font[5*(fig)+i];
      btm=0; top=0;
          // expend char    
      if(line & 64) {btm +=192;}
      if(line & 32) {btm +=48;}
      if(line & 16) {btm +=12;}           
      if(line & 8) {btm +=3;}

      if(line & 4) {top +=192;}
      if(line & 2) {top +=48;}
      if(line & 1) {top +=12;}        

       TinyI2C.write(top); //top page
       TinyI2C.write(btm);  //second page
       TinyI2C.write(top);
       TinyI2C.write(btm);
    }
    TinyI2C.stop();
        
    command(0x20);      // horizontal mode
    command(0x00);    
        
}
